// 组件通用属性与方法

import Emitter from "eventemitter3";
import type {
  RequestServiceObject,
  BaseOptions,
  FetchResult,
} from "@ahooksjs/use-request/lib/types";
export interface OptionsType {
  type?: string;
  _layout?: {
    _width: number,
    _height: number
  };
  _options?: any;
  _result?: FetchResult<any, any[]>;
  _children?: Array<ChildrenType>;
}

export interface ChildrenType extends OptionsType {
  id: string; // 子组件ID
}

export interface DatasourceType {
  title?: string; // 可以指定数据源名称
  type?: "API" | "JSON" | string;
  data?: any;
  service?: RequestServiceObject;
  options?: BaseOptions<any, any[]>;
}

export interface EventsType {
  [eventType: string]: {
    // 事件类型
    title: string; // 事件名称
    desc?: string; // 事件描述
    dimension?: string; // 绑定维度
    param?: string; // 影响参数
  };
}

// example
// events: {
//   clickCity: {
//     title: '选择城市',
//     desc: '当点击城市,会将城市绑定维度的值传递给对应参数',
//     dimension: '$[city]',
//     param: '${name}'
//   }
// }

export interface LayoutType {
  width: number;
  height: number;
}

export class Base extends Emitter {
  constructor(dom: string | Element, options: OptionsType) {
    super();
    this.dom = typeof dom === "string" ? document.querySelector(dom) : dom;
    this.instance = null;
    this.__resource = process.env.NODE_ENV === 'development' ? 'http://localhost:1234/statics' : `${window.XCOMP_PATH}/{{{hash}}}/{{{version}}}/statics`
  }
  
  public dom: Element | null;
  public instance: any;
  public __resource: string;
  static defaultValues = {
    options: {},
    datasource: [],
    events: {},
  };

  static schemas = {
    options: [],
    datasource: [],
    events: {},
  };

  public layout: LayoutType = {
    width: 200,
    height: 200,
  };

  public options: OptionsType = {};

  public datasource: DatasourceType[] = [];

  public events: EventsType = {};

  setOptions(options: OptionsType) {
    this.options = { ...this.options, ...options };
    if (this.options?._result?.loading === false) { // 仅当已获取数据后渲染
      this.render();
    }
  }

  /**
   * @description 渲染
   * @param info
   */
  render(options?: OptionsType) {
    console.log("没有指定渲染方式");
  }

  /**
   * @description 调整容器大小会修改options数据
   * @param layout
   */
  resize(layout: LayoutType) {
    this.layout = layout;
    this.options._layout = {
      _width: layout.width,
      _height: layout.height,
    };
    this.render(this.options);
  }

  /**
   *
   * 组件资源的销毁
   */
  destory() {
    console.log("没有指定销毁组件的方法");
  }

  /**
   * @description 更新数据 与 setOptions等不同 该update是更新后台传入得接口数据 如果不关心loading请求状态或者其他配置, 可直接等待updateData更新数据
   * @param data
   */
  updateData(data: any) {
    console.log("没有指定更新数据方法");
  }
}
// emiter(Base.prototype)

export default Base;
